@@ -4,10 +4,18 @@ from libifstate.address import Addresses
 from libifstate.routing import Tables, Rules
 from libifstate.sysctl import Sysctl
 from libifstate.parser import Parser
+from libifstate.tc import TC
+
+from pyroute2.netlink.exceptions import NetlinkError
 try:
     from libifstate.wireguard import WireGuard
 except ModuleNotFoundError:
+    # ignore missing plugin
     pass
+except NetlinkError:
+    # ignore plugin failure if kmod is not loaded
+    pass
+
 from libifstate.util import logger, ipr, IfStateLogging
 from libifstate.exception import FeatureMissingError, LinkCircularLinked, LinkNoConfigFound, NetlinkError, ParserValidationError
 from ipaddress import ip_network, ip_interface
@@ -19,7 +27,7 @@ import re
 import json
 import errno
 
-__version__ = "1.0.0"
+__version__ = "1.1.0"
 
 
 class IfState():
@@ -31,11 +39,13 @@ class IfState():
         self.tables = None
         self.rules = None
         self.sysctl = Sysctl()
+        self.tc = {}
         self.wireguard = {}
         self.features = {
             'link': True,
             'sysctl': os.access('/proc/sys/net', os.R_OK),
             'ethtool': not ethtool_path is None,
+            'tc': True,
             'wireguard': not globals().get("WireGuard") is None,
         }
 
@@ -94,6 +104,10 @@ class IfState():
             if 'sysctl' in ifstate:
                 self.sysctl.add(name, ifstate['sysctl'])
 
+            if 'tc' in ifstate:
+                self.tc[name] = TC(
+                    name, ifstate['tc'])
+
             if 'wireguard' in ifstate:
                 if not self.features['wireguard']:
                     raise FeatureMissingError("wireguard")
@@ -202,6 +216,16 @@ class IfState():
             if not retry:
                 break
 
+        if any(not x is None for x in self.tc.values()):
+            logger.info("\nconfiguring interface traffic control...")
+
+            for name, tc in self.tc.items():
+                if tc is None:
+                    logger.debug('skipped due to no tc settings', extra={
+                                 'iface': name})
+                else:
+                    tc.apply(do_apply)
+
         if any(not x is None for x in self.addresses.values()):
             logger.info("\nconfiguring interface ip addresses...")
             # add empty objects for unhandled interfaces
