@@ -8,14 +8,15 @@ class Parser(ABC):
             'ipaddr': [
                 'fe80::/10'
             ],
+            'ipaddr_dynamic': True,
             'ifname': [
+                r'^br-[\da-f]{12}',
                 r'^docker\d+',
                 r'^lo$',
                 r'^ppp\d+$',
                 r'^veth',
                 r'^virbr\d+',
-                r'^br-[\da-f]{12}',
-                r'^vrrp\d*\.\d+$',
+                r'^vrrp\d*\.\d+$'
             ],
             'routes': [
                 { 'proto': 1 },
